#include "audiolayerwidget.h"
#include <QComboBox>

AudioLayerWidget::AudioLayerWidget(QWidget *parent, int layer):
    QWidget(parent)
  , m_layer(layer)
  , m_suspendResumeButton(0)
{
    QVBoxLayout *layout = new QVBoxLayout;

    QVBoxLayout *playback = new QVBoxLayout;
    m_folderValue = new ClickableLabel;
    //m_folderValue->setMaximumWidth(160);
    m_folderValue->setAlignment(Qt::AlignHCenter);
    m_folderValue->setStyleSheet(
                "color: white;"
                "background-color: black;"
                );
    playback->addWidget(m_folderValue);
    m_fileValue = new ClickableLabel;
    connect(m_fileValue, SIGNAL(clicked()), this, SLOT(openMediaDialog()));
    connect(m_folderValue, SIGNAL(clicked()), this, SLOT(openMediaDialog()));
    //m_fileValue->setMaximumWidth(160);
    m_fileValue->setAlignment(Qt::AlignHCenter);
    m_fileValue->setStyleSheet(
                "color: white;"
                "background-color: black;"
                );
    playback->addWidget(m_fileValue);
    playback->setSpacing(0);
    playback->setContentsMargins(0, 0, 0, 0);
    layout->addLayout(playback);

    m_suspendResumeButton = new QPushButton(this);
    m_suspendResumeButton->setText(StatusStr[Status::Iddle]);
    //m_suspendResumeButton->setMaximumWidth(180);
    connect(m_suspendResumeButton, SIGNAL(clicked()), SLOT(toggleSuspendResume()));
    layout->addWidget(m_suspendResumeButton);

    m_progress = new QProgressBar(this);
    m_progress->setOrientation(Qt::Horizontal);
    m_progress->setRange(0, 0);
    m_progress->setValue(0);
    m_progress->setFormat("%v / %m");
    //m_progress->setMaximumWidth(180);
    layout->addWidget(m_progress);

    m_progressTime = new QTimeEdit;
    m_progressTime->setToolTip("Current Time");
    m_progressTime->setObjectName("Current Time");
    m_progressTime->setDisplayFormat("mm:ss:zz");
    m_progressTime->setReadOnly(true);
    m_progressTime->setButtonSymbols(QAbstractSpinBox::NoButtons);
    m_progressTime->setMinimumWidth(80);
    m_progressTime->setMaximumWidth(80);
    m_progressTime->setFocusPolicy(Qt::NoFocus);
    m_progressTime->setAlignment(Qt::AlignHCenter);
    m_progressTime->setContentsMargins(0,0,0,0);
    m_totalTimeValue = new QTimeEdit;
    m_totalTimeValue->setObjectName("Track Length");
    m_totalTimeValue->setToolTip("Track Length");
    m_totalTimeValue->setDisplayFormat("mm:ss:zzz");
    m_totalTimeValue->setReadOnly(true);
    m_totalTimeValue->setButtonSymbols(QAbstractSpinBox::NoButtons);
    m_totalTimeValue->setMinimumWidth(80);
    m_totalTimeValue->setFocusPolicy(Qt::NoFocus);
    m_totalTimeValue->setAlignment(Qt::AlignHCenter);
    m_totalTimeValue->setContentsMargins(0,0,0,0);
    QHBoxLayout *status = new QHBoxLayout;
    status->addWidget(m_progressTime);
    status->addWidget(m_totalTimeValue);
    layout->addLayout(status);
    QHBoxLayout *volumeBox = new QHBoxLayout;
    m_volume = new SliderGroup("Vol", 0 , 100, 2, NULL);
    volumeBox->addWidget(m_volume);
    connect(m_volume, SIGNAL(valueChanged(int)), this, SLOT(volumeChanged(int)));
    m_pan = new SliderGroup("Pan", 0 , 255, 0, NULL);
    volumeBox->addWidget(m_pan);
    connect(m_pan, SIGNAL(valueChanged(int)), this, SLOT(panChanged(int)));
    m_pitch = new SliderGroup("Pitch", 0 , 255, 0, NULL);
    volumeBox->addWidget(m_pitch);
    volumeBox->setSpacing(0);
    volumeBox->setContentsMargins(0, 0, 0, 0);
    connect(m_pitch, SIGNAL(valueChanged(int)), this, SLOT(pitchChanged(int)));
    layout->addLayout(volumeBox);
    layout->setAlignment(Qt::AlignHCenter);
    layout->setSpacing(0);
    layout->setContentsMargins(2, 2, 2, 2);
    this->setLayout(layout);
}

AudioLayerWidget::~AudioLayerWidget()
{

}

// From UI.
void AudioLayerWidget::volumeChanged(int value)
{
    emit(uiSliderChanged(m_layer, Slider::Volume, value));
}

void AudioLayerWidget::panChanged(int value)
{
    emit(uiSliderChanged(m_layer, Slider::Pan, value));
}

void AudioLayerWidget::pitchChanged(int value)
{
    emit(uiSliderChanged(m_layer, Slider::Pitch, value));
}

void AudioLayerWidget::toggleSuspendResume()
{
    switch (m_status) {
        case Status::PlayingLoop:
        case Status::PlayingOnce:
            this->setPlaybackStatus(Status::Paused);
            emit uiPlaybackChanged(m_layer, Status::Paused);
            break;
        case Status::Paused:
        case Status::Stopped:
            this->setPlaybackStatus(Status::PlayingLoop);
            emit uiPlaybackChanged(m_layer, Status::PlayingLoop);
        case Status::Iddle:
            break;
    }
}

void AudioLayerWidget::openMediaDialog()
{
    QFileDialog dialog(this);
    dialog.setFileMode(QFileDialog::ExistingFile);
    dialog.setNameFilter(tr("Sound Tracks (*.mp3 *.mp3 *.flac *.wav"));
    dialog.setViewMode(QFileDialog::Detail);
    dialog.setDirectory(Settings::getInstance()->getPathMedia());
    if (!dialog.exec())
        return;
    QStringList fileNames;
    fileNames = dialog.selectedFiles();
    emit uiLoadMedia(m_layer, fileNames.at(0));
    this->setMediaFile(fileNames.at(0));
}

// from DMX signals
void AudioLayerWidget::setVol(float vol)
{
    m_volume->blockSignals(true);
    m_volume->setValue(vol);
    m_volume->blockSignals(false);
}

void AudioLayerWidget::setPan(int pan)
{
    m_pan->blockSignals(true);
    m_pan->setValue(pan);
    m_pan->blockSignals(false);
}

void AudioLayerWidget::setPitch(int pitch)
{
    m_pitch->blockSignals(true);
    m_pitch->setValue(pitch);
    m_pitch->blockSignals(false);
}

void AudioLayerWidget::setMediaFile(QString file)
{
    QStringList list = file.split("/");
    int size = list.size();
    if (size >= 2) {
       m_folderValue->setText(list.at(size - 2));
       m_fileValue->setText(list.at(size - 1));
    }
    this->setPlaybackStatus(Status::Stopped);
}

void AudioLayerWidget::setPlaybackStatus(Status s)
{
    Status status = static_cast<Status>(s);
    m_suspendResumeButton->blockSignals(true);
    m_status = status;
    //if (status == Status::Stopped)
    //    refreshCurrentTime(0);
    m_suspendResumeButton->setText(StatusStr[status]);
    m_suspendResumeButton->blockSignals(false);
}

void AudioLayerWidget::setDuration(float dur)
{
    m_progress->blockSignals(true);
    m_progressTime->blockSignals(true);
    m_totalTimeValue->blockSignals(true);
    m_progress->setRange(0, dur);
    m_progress->setValue(0);
    m_progressTime->setTime(QTime::fromMSecsSinceStartOfDay(0));
    m_totalTimeValue->setTime(QTime::fromMSecsSinceStartOfDay(dur));
    m_progress->blockSignals(false);
    m_progressTime->blockSignals(false);
    m_totalTimeValue->blockSignals(false);
}

void AudioLayerWidget::setCurrentTime(float progress)
{
     progress *= 1000;
     m_progress->blockSignals(true);
     m_progressTime->blockSignals(true);
     m_progress->setValue(progress);
     m_progressTime->setTime(QTime::fromMSecsSinceStartOfDay(progress));
     m_progress->blockSignals(false);
     m_progressTime->blockSignals(false);
}
